@@ -1,8 +1,8 @@
 //
-// socket_option.hpp
-// ~~~~~~~~~~~~~~~~~
+// detail/socket_option.hpp
+// ~~~~~~~~~~~~~~~~~~~~~~~~
 //
-// Copyright (c) 2003-2008 Christopher M. Kohlhoff (chris at kohlhoff dot com)
+// Copyright (c) 2003-2018 Christopher M. Kohlhoff (chris at kohlhoff dot com)
 //
 // Distributed under the Boost Software License, Version 1.0. (See accompanying
 // file LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)
@@ -15,17 +15,16 @@
 # pragma once
 #endif // defined(_MSC_VER) && (_MSC_VER >= 1200)
 
-#include "asio/detail/push_options.hpp"
-
-#include "asio/detail/push_options.hpp"
+#include "asio/detail/config.hpp"
 #include <cstddef>
 #include <cstring>
-#include <boost/config.hpp>
-#include "asio/detail/pop_options.hpp"
-
-#include "asio/ip/address.hpp"
+#include <stdexcept>
 #include "asio/detail/socket_ops.hpp"
 #include "asio/detail/socket_types.hpp"
+#include "asio/detail/throw_exception.hpp"
+#include "asio/ip/address.hpp"
+
+#include "asio/detail/push_options.hpp"
 
 namespace asio {
 namespace ip {
@@ -141,8 +140,8 @@ public:
     {
       if (s != sizeof(ipv6_value_))
       {
-        throw std::length_error(
-            "multicast_enable_loopback socket option resize");
+        std::length_error ex("multicast_enable_loopback socket option resize");
+        asio::detail::throw_exception(ex);
       }
       ipv4_value_ = ipv6_value_ ? 1 : 0;
     }
@@ -150,8 +149,8 @@ public:
     {
       if (s != sizeof(ipv4_value_))
       {
-        throw std::length_error(
-            "multicast_enable_loopback socket option resize");
+        std::length_error ex("multicast_enable_loopback socket option resize");
+        asio::detail::throw_exception(ex);
       }
       ipv6_value_ = ipv4_value_ ? 1 : 0;
     }
@@ -236,7 +235,10 @@ public:
   void resize(const Protocol&, std::size_t s)
   {
     if (s != sizeof(value_))
-      throw std::length_error("unicast hops socket option resize");
+    {
+      std::length_error ex("unicast hops socket option resize");
+      asio::detail::throw_exception(ex);
+    }
 #if defined(__hpux)
     if (value_ < 0)
       value_ = value_ & 0xFF;
@@ -252,7 +254,7 @@ template <int IPv4_Level, int IPv4_Name, int IPv6_Level, int IPv6_Name>
 class multicast_hops
 {
 public:
-#if defined(BOOST_WINDOWS) && defined(UNDER_CE)
+#if defined(ASIO_WINDOWS) && defined(UNDER_CE)
   typedef int ipv4_value_type;
 #else
   typedef unsigned char ipv4_value_type;
@@ -270,7 +272,10 @@ public:
   explicit multicast_hops(int v)
   {
     if (v < 0 || v > 255)
-      throw std::out_of_range("multicast hops value out of range");
+    {
+      std::out_of_range ex("multicast hops value out of range");
+      asio::detail::throw_exception(ex);
+    }
     ipv4_value_ = (ipv4_value_type)v;
     ipv6_value_ = v;
   }
@@ -279,7 +284,10 @@ public:
   multicast_hops& operator=(int v)
   {
     if (v < 0 || v > 255)
-      throw std::out_of_range("multicast hops value out of range");
+    {
+      std::out_of_range ex("multicast hops value out of range");
+      asio::detail::throw_exception(ex);
+    }
     ipv4_value_ = (ipv4_value_type)v;
     ipv6_value_ = v;
     return *this;
@@ -343,7 +351,10 @@ public:
     if (protocol.family() == PF_INET6)
     {
       if (s != sizeof(ipv6_value_))
-        throw std::length_error("multicast hops socket option resize");
+      {
+        std::length_error ex("multicast hops socket option resize");
+        asio::detail::throw_exception(ex);
+      }
       if (ipv6_value_ < 0)
         ipv4_value_ = 0;
       else if (ipv6_value_ > 255)
@@ -354,7 +365,10 @@ public:
     else
     {
       if (s != sizeof(ipv4_value_))
-        throw std::length_error("multicast hops socket option resize");
+      {
+        std::length_error ex("multicast hops socket option resize");
+        asio::detail::throw_exception(ex);
+      }
       ipv6_value_ = ipv4_value_;
     }
   }
@@ -371,87 +385,61 @@ class multicast_request
 public:
   // Default constructor.
   multicast_request()
+    : ipv4_value_(), // Zero-initialisation gives the "any" address.
+      ipv6_value_() // Zero-initialisation gives the "any" address.
   {
-    ipv4_value_.imr_multiaddr.s_addr =
-      asio::detail::socket_ops::host_to_network_long(
-          asio::ip::address_v4::any().to_ulong());
-    ipv4_value_.imr_interface.s_addr =
-      asio::detail::socket_ops::host_to_network_long(
-          asio::ip::address_v4::any().to_ulong());
-
-    asio::detail::in6_addr_type tmp_addr = IN6ADDR_ANY_INIT;
-    ipv6_value_.ipv6mr_multiaddr = tmp_addr;
-    ipv6_value_.ipv6mr_interface = 0;
   }
 
   // Construct with multicast address only.
-  explicit multicast_request(const asio::ip::address& multicast_address)
+  explicit multicast_request(const address& multicast_address)
+    : ipv4_value_(), // Zero-initialisation gives the "any" address.
+      ipv6_value_() // Zero-initialisation gives the "any" address.
   {
     if (multicast_address.is_v6())
     {
-      ipv4_value_.imr_multiaddr.s_addr =
-        asio::detail::socket_ops::host_to_network_long(
-            asio::ip::address_v4::any().to_ulong());
-      ipv4_value_.imr_interface.s_addr =
-        asio::detail::socket_ops::host_to_network_long(
-            asio::ip::address_v4::any().to_ulong());
-
       using namespace std; // For memcpy.
-      asio::ip::address_v6 ipv6_address = multicast_address.to_v6();
-      asio::ip::address_v6::bytes_type bytes = ipv6_address.to_bytes();
-      memcpy(ipv6_value_.ipv6mr_multiaddr.s6_addr, bytes.elems, 16);
-      ipv6_value_.ipv6mr_interface = 0;
+      address_v6 ipv6_address = multicast_address.to_v6();
+      address_v6::bytes_type bytes = ipv6_address.to_bytes();
+      memcpy(ipv6_value_.ipv6mr_multiaddr.s6_addr, bytes.data(), 16);
+      ipv6_value_.ipv6mr_interface = ipv6_address.scope_id();
     }
     else
     {
       ipv4_value_.imr_multiaddr.s_addr =
         asio::detail::socket_ops::host_to_network_long(
-            multicast_address.to_v4().to_ulong());
+            multicast_address.to_v4().to_uint());
       ipv4_value_.imr_interface.s_addr =
         asio::detail::socket_ops::host_to_network_long(
-            asio::ip::address_v4::any().to_ulong());
-
-      asio::detail::in6_addr_type tmp_addr = IN6ADDR_ANY_INIT;
-      ipv6_value_.ipv6mr_multiaddr = tmp_addr;
-      ipv6_value_.ipv6mr_interface = 0;
+            address_v4::any().to_uint());
     }
   }
 
   // Construct with multicast address and IPv4 address specifying an interface.
-  explicit multicast_request(
-      const asio::ip::address_v4& multicast_address,
-      const asio::ip::address_v4& network_interface
-        = asio::ip::address_v4::any())
+  explicit multicast_request(const address_v4& multicast_address,
+      const address_v4& network_interface = address_v4::any())
+    : ipv6_value_() // Zero-initialisation gives the "any" address.
   {
     ipv4_value_.imr_multiaddr.s_addr =
       asio::detail::socket_ops::host_to_network_long(
-          multicast_address.to_ulong());
+          multicast_address.to_uint());
     ipv4_value_.imr_interface.s_addr =
       asio::detail::socket_ops::host_to_network_long(
-          network_interface.to_ulong());
-
-    asio::detail::in6_addr_type tmp_addr = IN6ADDR_ANY_INIT;
-    ipv6_value_.ipv6mr_multiaddr = tmp_addr;
-    ipv6_value_.ipv6mr_interface = 0;
+          network_interface.to_uint());
   }
 
   // Construct with multicast address and IPv6 network interface index.
   explicit multicast_request(
-      const asio::ip::address_v6& multicast_address,
+      const address_v6& multicast_address,
       unsigned long network_interface = 0)
+    : ipv4_value_() // Zero-initialisation gives the "any" address.
   {
-    ipv4_value_.imr_multiaddr.s_addr =
-      asio::detail::socket_ops::host_to_network_long(
-          asio::ip::address_v4::any().to_ulong());
-    ipv4_value_.imr_interface.s_addr =
-      asio::detail::socket_ops::host_to_network_long(
-          asio::ip::address_v4::any().to_ulong());
-
     using namespace std; // For memcpy.
-    asio::ip::address_v6::bytes_type bytes =
-      multicast_address.to_bytes();
-    memcpy(ipv6_value_.ipv6mr_multiaddr.s6_addr, bytes.elems, 16);
-    ipv6_value_.ipv6mr_interface = network_interface;
+    address_v6::bytes_type bytes = multicast_address.to_bytes();
+    memcpy(ipv6_value_.ipv6mr_multiaddr.s6_addr, bytes.data(), 16);
+    if (network_interface)
+      ipv6_value_.ipv6mr_interface = network_interface;
+    else
+      ipv6_value_.ipv6mr_interface = multicast_address.scope_id();
   }
 
   // Get the level of the socket option.
@@ -505,16 +493,16 @@ public:
   {
     ipv4_value_.s_addr =
       asio::detail::socket_ops::host_to_network_long(
-          asio::ip::address_v4::any().to_ulong());
+          address_v4::any().to_uint());
     ipv6_value_ = 0;
   }
 
   // Construct with IPv4 interface.
-  explicit network_interface(const asio::ip::address_v4& ipv4_interface)
+  explicit network_interface(const address_v4& ipv4_interface)
   {
     ipv4_value_.s_addr =
       asio::detail::socket_ops::host_to_network_long(
-          ipv4_interface.to_ulong());
+          ipv4_interface.to_uint());
     ipv6_value_ = 0;
   }
 
@@ -523,7 +511,7 @@ public:
   {
     ipv4_value_.s_addr =
       asio::detail::socket_ops::host_to_network_long(
-          asio::ip::address_v4::any().to_ulong());
+          address_v4::any().to_uint());
     ipv6_value_ = ipv6_interface;
   }
 
