@@ -2,7 +2,7 @@
 // buffers_iterator.cpp
 // ~~~~~~~~~~~~~~~~~~~~
 //
-// Copyright (c) 2003-2011 Christopher M. Kohlhoff (chris at kohlhoff dot com)
+// Copyright (c) 2003-2018 Christopher M. Kohlhoff (chris at kohlhoff dot com)
 //
 // Distributed under the Boost Software License, Version 1.0. (See accompanying
 // file LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)
@@ -16,10 +16,17 @@
 // Test that header file is self-contained.
 #include "asio/buffers_iterator.hpp"
 
-#include <boost/array.hpp>
 #include "asio/buffer.hpp"
 #include "unit_test.hpp"
 
+#if defined(ASIO_HAS_BOOST_ARRAY)
+# include <boost/array.hpp>
+#endif // defined(ASIO_HAS_BOOST_ARRAY)
+
+#if defined(ASIO_HAS_STD_ARRAY)
+# include <array>
+#endif // defined(ASIO_HAS_STD_ARRAY)
+
 //------------------------------------------------------------------------------
 
 // buffers_iterator_compile test
@@ -29,7 +36,11 @@
 
 namespace buffers_iterator_compile {
 
+#if defined(ASIO_HAS_BOOST_ARRAY)
 using boost::array;
+#elif defined(ASIO_HAS_STD_ARRAY)
+using std::array;
+#endif // defined(ASIO_HAS_BOOST_ARRAY)
 using std::vector;
 using namespace asio;
 
@@ -39,11 +50,11 @@ void test()
   {
     char data1[16], data2[16];
     const char cdata1[16] = "", cdata2[16] = "";
-    mutable_buffers_1 mb1 = buffer(data1);
+    mutable_buffer mb1 = buffer(data1);
     array<mutable_buffer, 2> mb2 = {{ buffer(data1), buffer(data2) }};
     std::vector<mutable_buffer> mb3;
     mb3.push_back(buffer(data1));
-    const_buffers_1 cb1 = buffer(cdata1);
+    const_buffer cb1 = buffer(cdata1);
     array<const_buffer, 2> cb2 = {{ buffer(cdata1), buffer(cdata2) }};
     vector<const_buffer> cb3;
     cb3.push_back(buffer(cdata1));
@@ -51,23 +62,23 @@ void test()
 
     // buffers_iterator constructors.
 
-    buffers_iterator<mutable_buffers_1, char> bi1;
-    buffers_iterator<mutable_buffers_1, const char> bi2;
+    buffers_iterator<mutable_buffer, char> bi1;
+    buffers_iterator<mutable_buffer, const char> bi2;
     buffers_iterator<array<mutable_buffer, 2>, char> bi3;
     buffers_iterator<array<mutable_buffer, 2>, const char> bi4;
     buffers_iterator<vector<mutable_buffer>, char> bi5;
     buffers_iterator<vector<mutable_buffer>, const char> bi6;
-    buffers_iterator<const_buffers_1, char> bi7;
-    buffers_iterator<const_buffers_1, const char> bi8;
+    buffers_iterator<const_buffer, char> bi7;
+    buffers_iterator<const_buffer, const char> bi8;
     buffers_iterator<array<const_buffer, 2>, char> bi9;
     buffers_iterator<array<const_buffer, 2>, const char> bi10;
     buffers_iterator<vector<const_buffer>, char> bi11;
     buffers_iterator<vector<const_buffer>, const char> bi12;
 
-    buffers_iterator<mutable_buffers_1, char> bi13(
-        buffers_iterator<mutable_buffers_1, char>::begin(mb1));
-    buffers_iterator<mutable_buffers_1, const char> bi14(
-        buffers_iterator<mutable_buffers_1, const char>::begin(mb1));
+    buffers_iterator<mutable_buffer, char> bi13(
+        buffers_iterator<mutable_buffer, char>::begin(mb1));
+    buffers_iterator<mutable_buffer, const char> bi14(
+        buffers_iterator<mutable_buffer, const char>::begin(mb1));
     buffers_iterator<array<mutable_buffer, 2>, char> bi15(
         buffers_iterator<array<mutable_buffer, 2>, char>::begin(mb2));
     buffers_iterator<array<mutable_buffer, 2>, const char> bi16(
@@ -76,10 +87,10 @@ void test()
         buffers_iterator<vector<mutable_buffer>, char>::begin(mb3));
     buffers_iterator<vector<mutable_buffer>, const char> bi18(
         buffers_iterator<vector<mutable_buffer>, const char>::begin(mb3));
-    buffers_iterator<const_buffers_1, char> bi19(
-        buffers_iterator<const_buffers_1, char>::begin(cb1));
-    buffers_iterator<const_buffers_1, const char> bi20(
-        buffers_iterator<const_buffers_1, const char>::begin(cb1));
+    buffers_iterator<const_buffer, char> bi19(
+        buffers_iterator<const_buffer, char>::begin(cb1));
+    buffers_iterator<const_buffer, const char> bi20(
+        buffers_iterator<const_buffer, const char>::begin(cb1));
     buffers_iterator<array<const_buffer, 2>, char> bi21(
         buffers_iterator<array<const_buffer, 2>, char>::begin(cb2));
     buffers_iterator<array<const_buffer, 2>, const char> bi22(
@@ -91,27 +102,27 @@ void test()
 
     // buffers_iterator member functions.
 
-    bi1 = buffers_iterator<mutable_buffers_1, char>::begin(mb1);
-    bi2 = buffers_iterator<mutable_buffers_1, const char>::begin(mb1);
+    bi1 = buffers_iterator<mutable_buffer, char>::begin(mb1);
+    bi2 = buffers_iterator<mutable_buffer, const char>::begin(mb1);
     bi3 = buffers_iterator<array<mutable_buffer, 2>, char>::begin(mb2);
     bi4 = buffers_iterator<array<mutable_buffer, 2>, const char>::begin(mb2);
     bi5 = buffers_iterator<vector<mutable_buffer>, char>::begin(mb3);
     bi6 = buffers_iterator<vector<mutable_buffer>, const char>::begin(mb3);
-    bi7 = buffers_iterator<const_buffers_1, char>::begin(cb1);
-    bi8 = buffers_iterator<const_buffers_1, const char>::begin(cb1);
+    bi7 = buffers_iterator<const_buffer, char>::begin(cb1);
+    bi8 = buffers_iterator<const_buffer, const char>::begin(cb1);
     bi9 = buffers_iterator<array<const_buffer, 2>, char>::begin(cb2);
     bi10 = buffers_iterator<array<const_buffer, 2>, const char>::begin(cb2);
     bi11 = buffers_iterator<vector<const_buffer>, char>::begin(cb3);
     bi12 = buffers_iterator<vector<const_buffer>, const char>::begin(cb3);
 
-    bi1 = buffers_iterator<mutable_buffers_1, char>::end(mb1);
-    bi2 = buffers_iterator<mutable_buffers_1, const char>::end(mb1);
+    bi1 = buffers_iterator<mutable_buffer, char>::end(mb1);
+    bi2 = buffers_iterator<mutable_buffer, const char>::end(mb1);
     bi3 = buffers_iterator<array<mutable_buffer, 2>, char>::end(mb2);
     bi4 = buffers_iterator<array<mutable_buffer, 2>, const char>::end(mb2);
     bi5 = buffers_iterator<vector<mutable_buffer>, char>::end(mb3);
     bi6 = buffers_iterator<vector<mutable_buffer>, const char>::end(mb3);
-    bi7 = buffers_iterator<const_buffers_1, char>::end(cb1);
-    bi8 = buffers_iterator<const_buffers_1, const char>::end(cb1);
+    bi7 = buffers_iterator<const_buffer, char>::end(cb1);
+    bi8 = buffers_iterator<const_buffer, const char>::end(cb1);
     bi9 = buffers_iterator<array<const_buffer, 2>, char>::end(cb2);
     bi10 = buffers_iterator<array<const_buffer, 2>, const char>::end(cb2);
     bi11 = buffers_iterator<vector<const_buffer>, char>::end(cb3);
@@ -274,9 +285,8 @@ void test()
 
 //------------------------------------------------------------------------------
 
-test_suite* init_unit_test_suite(int, char*[])
-{
-  test_suite* test = BOOST_TEST_SUITE("buffers_iterator");
-  test->add(BOOST_TEST_CASE(&buffers_iterator_compile::test));
-  return test;
-}
+ASIO_TEST_SUITE
+(
+  "buffers_iterator",
+  ASIO_TEST_CASE(buffers_iterator_compile::test)
+)
