@@ -2,7 +2,7 @@
 // stream_protocol.cpp
 // ~~~~~~~~~~~~~~~~~~~
 //
-// Copyright (c) 2003-2011 Christopher M. Kohlhoff (chris at kohlhoff dot com)
+// Copyright (c) 2003-2018 Christopher M. Kohlhoff (chris at kohlhoff dot com)
 //
 // Distributed under the Boost Software License, Version 1.0. (See accompanying
 // file LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)
@@ -16,9 +16,8 @@
 // Test that header file is self-contained.
 #include "asio/local/stream_protocol.hpp"
 
-#include <boost/bind.hpp>
 #include <cstring>
-#include "asio/io_service.hpp"
+#include "asio/io_context.hpp"
 #include "../unit_test.hpp"
 
 //------------------------------------------------------------------------------
@@ -60,7 +59,7 @@ void test()
 
   try
   {
-    io_service ios;
+    io_context ioc;
     char mutable_char_buffer[128] = "";
     const char const_char_buffer[128] = "";
     socket_base::message_flags in_flags = 0;
@@ -70,16 +69,21 @@ void test()
 
     // basic_stream_socket constructors.
 
-    sp::socket socket1(ios);
-    sp::socket socket2(ios, sp());
-    sp::socket socket3(ios, sp::endpoint(""));
+    sp::socket socket1(ioc);
+    sp::socket socket2(ioc, sp());
+    sp::socket socket3(ioc, sp::endpoint(""));
     int native_socket1 = ::socket(AF_UNIX, SOCK_STREAM, 0);
-    sp::socket socket4(ios, sp(), native_socket1);
+    sp::socket socket4(ioc, sp(), native_socket1);
 
     // basic_io_object functions.
 
-    io_service& ios_ref = socket1.io_service();
-    (void)ios_ref;
+#if !defined(ASIO_NO_DEPRECATED)
+    io_context& ioc_ref = socket1.get_io_context();
+    (void)ioc_ref;
+#endif // !defined(ASIO_NO_DEPRECATED)
+
+    sp::socket::executor_type ex = socket1.get_executor();
+    (void)ex;
 
     // basic_socket functions.
 
@@ -100,7 +104,7 @@ void test()
     socket1.close();
     socket1.close(ec);
 
-    sp::socket::native_type native_socket4 = socket1.native();
+    sp::socket::native_handle_type native_socket4 = socket1.native_handle();
     (void)native_socket4;
 
     socket1.cancel();
@@ -202,9 +206,8 @@ void test()
 
 //------------------------------------------------------------------------------
 
-test_suite* init_unit_test_suite(int, char*[])
-{
-  test_suite* test = BOOST_TEST_SUITE("local/stream_protocol");
-  test->add(BOOST_TEST_CASE(&local_stream_protocol_socket_compile::test));
-  return test;
-}
+ASIO_TEST_SUITE
+(
+  "local/stream_protocol",
+  ASIO_TEST_CASE(local_stream_protocol_socket_compile::test)
+)
