@@ -2,7 +2,7 @@
 // read_until.cpp
 // ~~~~~~~~~~~~~~
 //
-// Copyright (c) 2003-2011 Christopher M. Kohlhoff (chris at kohlhoff dot com)
+// Copyright (c) 2003-2018 Christopher M. Kohlhoff (chris at kohlhoff dot com)
 //
 // Distributed under the Boost Software License, Version 1.0. (See accompanying
 // file LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)
@@ -16,38 +16,42 @@
 // Test that header file is self-contained.
 #include "asio/read_until.hpp"
 
-#include <boost/bind.hpp>
-#include <boost/noncopyable.hpp>
 #include <cstring>
-#include "asio/io_service.hpp"
-#include "asio/placeholders.hpp"
+#include "archetypes/async_result.hpp"
+#include "asio/io_context.hpp"
+#include "asio/post.hpp"
 #include "asio/streambuf.hpp"
 #include "unit_test.hpp"
 
+#if defined(ASIO_HAS_BOOST_BIND)
+# include <boost/bind.hpp>
+#else // defined(ASIO_HAS_BOOST_BIND)
+# include <functional>
+#endif // defined(ASIO_HAS_BOOST_BIND)
+
 class test_stream
-  : private boost::noncopyable
 {
 public:
-  typedef asio::io_service io_service_type;
+  typedef asio::io_context::executor_type executor_type;
 
-  test_stream(asio::io_service& io_service)
-    : io_service_(io_service),
+  test_stream(asio::io_context& io_context)
+    : io_context_(io_context),
       length_(0),
       position_(0),
       next_read_length_(0)
   {
   }
 
-  io_service_type& get_io_service()
+  executor_type get_executor() ASIO_NOEXCEPT
   {
-    return io_service_;
+    return io_context_.get_executor();
   }
 
   void reset(const void* data, size_t length)
   {
     using namespace std; // For memcpy.
 
-    BOOST_CHECK(length <= max_length);
+    ASIO_CHECK(length <= max_length);
 
     memcpy(data_, data, length);
     length_ = length;
@@ -63,27 +67,11 @@ public:
   template <typename Mutable_Buffers>
   size_t read_some(const Mutable_Buffers& buffers)
   {
-    using namespace std; // For memcpy.
-
-    size_t total_length = 0;
-
-    typename Mutable_Buffers::const_iterator iter = buffers.begin();
-    typename Mutable_Buffers::const_iterator end = buffers.end();
-    for (; iter != end && total_length < next_read_length_; ++iter)
-    {
-      size_t length = asio::buffer_size(*iter);
-      if (length > length_ - position_)
-        length = length_ - position_;
-
-      if (length > next_read_length_ - total_length)
-        length = next_read_length_ - total_length;
-
-      memcpy(asio::buffer_cast<void*>(*iter), data_ + position_, length);
-      position_ += length;
-      total_length += length;
-    }
-
-    return total_length;
+    size_t n = asio::buffer_copy(buffers,
+        asio::buffer(data_, length_) + position_,
+        next_read_length_);
+    position_ += n;
+    return n;
   }
 
   template <typename Mutable_Buffers>
@@ -98,12 +86,14 @@ public:
   void async_read_some(const Mutable_Buffers& buffers, Handler handler)
   {
     size_t bytes_transferred = read_some(buffers);
-    io_service_.post(asio::detail::bind_handler(
-          handler, asio::error_code(), bytes_transferred));
+    asio::post(get_executor(),
+        asio::detail::bind_handler(
+          ASIO_MOVE_CAST(Handler)(handler),
+          asio::error_code(), bytes_transferred));
   }
 
 private:
-  io_service_type& io_service_;
+  asio::io_context& io_context_;
   enum { max_length = 8192 };
   char data_[max_length];
   size_t length_;
@@ -116,8 +106,8 @@ static const char read_data[]
 
 void test_char_read_until()
 {
-  asio::io_service ios;
-  test_stream s(ios);
+  asio::io_context ioc;
+  test_stream s(ioc);
   asio::streambuf sb1;
   asio::streambuf sb2(25);
   asio::error_code ec;
@@ -125,85 +115,85 @@ void test_char_read_until()
   s.reset(read_data, sizeof(read_data));
   sb1.consume(sb1.size());
   std::size_t length = asio::read_until(s, sb1, 'Z');
-  BOOST_CHECK(length == 26);
+  ASIO_CHECK(length == 26);
 
   s.reset(read_data, sizeof(read_data));
   s.next_read_length(1);
   sb1.consume(sb1.size());
   length = asio::read_until(s, sb1, 'Z');
-  BOOST_CHECK(length == 26);
+  ASIO_CHECK(length == 26);
 
   s.reset(read_data, sizeof(read_data));
   s.next_read_length(10);
   sb1.consume(sb1.size());
   length = asio::read_until(s, sb1, 'Z');
-  BOOST_CHECK(length == 26);
+  ASIO_CHECK(length == 26);
 
   s.reset(read_data, sizeof(read_data));
   sb1.consume(sb1.size());
   length = asio::read_until(s, sb1, 'Z', ec);
-  BOOST_CHECK(!ec);
-  BOOST_CHECK(length == 26);
+  ASIO_CHECK(!ec);
+  ASIO_CHECK(length == 26);
 
   s.reset(read_data, sizeof(read_data));
   s.next_read_length(1);
   sb1.consume(sb1.size());
   length = asio::read_until(s, sb1, 'Z', ec);
-  BOOST_CHECK(!ec);
-  BOOST_CHECK(length == 26);
+  ASIO_CHECK(!ec);
+  ASIO_CHECK(length == 26);
 
   s.reset(read_data, sizeof(read_data));
   s.next_read_length(10);
   sb1.consume(sb1.size());
   length = asio::read_until(s, sb1, 'Z', ec);
-  BOOST_CHECK(!ec);
-  BOOST_CHECK(length == 26);
+  ASIO_CHECK(!ec);
+  ASIO_CHECK(length == 26);
 
   s.reset(read_data, sizeof(read_data));
   sb2.consume(sb2.size());
   length = asio::read_until(s, sb2, 'Z', ec);
-  BOOST_CHECK(ec == asio::error::not_found);
-  BOOST_CHECK(length == 0);
+  ASIO_CHECK(ec == asio::error::not_found);
+  ASIO_CHECK(length == 0);
 
   s.reset(read_data, sizeof(read_data));
   s.next_read_length(1);
   sb2.consume(sb2.size());
   length = asio::read_until(s, sb2, 'Z', ec);
-  BOOST_CHECK(ec == asio::error::not_found);
-  BOOST_CHECK(length == 0);
+  ASIO_CHECK(ec == asio::error::not_found);
+  ASIO_CHECK(length == 0);
 
   s.reset(read_data, sizeof(read_data));
   s.next_read_length(10);
   sb2.consume(sb2.size());
   length = asio::read_until(s, sb2, 'Z', ec);
-  BOOST_CHECK(ec == asio::error::not_found);
-  BOOST_CHECK(length == 0);
+  ASIO_CHECK(ec == asio::error::not_found);
+  ASIO_CHECK(length == 0);
 
   s.reset(read_data, sizeof(read_data));
   sb2.consume(sb2.size());
   length = asio::read_until(s, sb2, 'Y', ec);
-  BOOST_CHECK(!ec);
-  BOOST_CHECK(length == 25);
+  ASIO_CHECK(!ec);
+  ASIO_CHECK(length == 25);
 
   s.reset(read_data, sizeof(read_data));
   s.next_read_length(1);
   sb2.consume(sb2.size());
   length = asio::read_until(s, sb2, 'Y', ec);
-  BOOST_CHECK(!ec);
-  BOOST_CHECK(length == 25);
+  ASIO_CHECK(!ec);
+  ASIO_CHECK(length == 25);
 
   s.reset(read_data, sizeof(read_data));
   s.next_read_length(10);
   sb2.consume(sb2.size());
   length = asio::read_until(s, sb2, 'Y', ec);
-  BOOST_CHECK(!ec);
-  BOOST_CHECK(length == 25);
+  ASIO_CHECK(!ec);
+  ASIO_CHECK(length == 25);
 }
 
 void test_string_read_until()
 {
-  asio::io_service ios;
-  test_stream s(ios);
+  asio::io_context ioc;
+  test_stream s(ioc);
   asio::streambuf sb1;
   asio::streambuf sb2(25);
   asio::error_code ec;
@@ -211,79 +201,79 @@ void test_string_read_until()
   s.reset(read_data, sizeof(read_data));
   sb1.consume(sb1.size());
   std::size_t length = asio::read_until(s, sb1, "XYZ");
-  BOOST_CHECK(length == 26);
+  ASIO_CHECK(length == 26);
 
   s.reset(read_data, sizeof(read_data));
   s.next_read_length(1);
   sb1.consume(sb1.size());
   length = asio::read_until(s, sb1, "XYZ");
-  BOOST_CHECK(length == 26);
+  ASIO_CHECK(length == 26);
 
   s.reset(read_data, sizeof(read_data));
   s.next_read_length(10);
   sb1.consume(sb1.size());
   length = asio::read_until(s, sb1, "XYZ");
-  BOOST_CHECK(length == 26);
+  ASIO_CHECK(length == 26);
 
   s.reset(read_data, sizeof(read_data));
   sb1.consume(sb1.size());
   length = asio::read_until(s, sb1, "XYZ", ec);
-  BOOST_CHECK(!ec);
-  BOOST_CHECK(length == 26);
+  ASIO_CHECK(!ec);
+  ASIO_CHECK(length == 26);
 
   s.reset(read_data, sizeof(read_data));
   s.next_read_length(1);
   sb1.consume(sb1.size());
   length = asio::read_until(s, sb1, "XYZ", ec);
-  BOOST_CHECK(!ec);
-  BOOST_CHECK(length == 26);
+  ASIO_CHECK(!ec);
+  ASIO_CHECK(length == 26);
 
   s.reset(read_data, sizeof(read_data));
   s.next_read_length(10);
   sb1.consume(sb1.size());
   length = asio::read_until(s, sb1, "XYZ", ec);
-  BOOST_CHECK(!ec);
-  BOOST_CHECK(length == 26);
+  ASIO_CHECK(!ec);
+  ASIO_CHECK(length == 26);
 
   s.reset(read_data, sizeof(read_data));
   sb2.consume(sb2.size());
   length = asio::read_until(s, sb2, "XYZ", ec);
-  BOOST_CHECK(ec == asio::error::not_found);
-  BOOST_CHECK(length == 0);
+  ASIO_CHECK(ec == asio::error::not_found);
+  ASIO_CHECK(length == 0);
 
   s.reset(read_data, sizeof(read_data));
   s.next_read_length(1);
   sb2.consume(sb2.size());
   length = asio::read_until(s, sb2, "XYZ", ec);
-  BOOST_CHECK(ec == asio::error::not_found);
-  BOOST_CHECK(length == 0);
+  ASIO_CHECK(ec == asio::error::not_found);
+  ASIO_CHECK(length == 0);
 
   s.reset(read_data, sizeof(read_data));
   s.next_read_length(10);
   sb2.consume(sb2.size());
   length = asio::read_until(s, sb2, "XYZ", ec);
-  BOOST_CHECK(ec == asio::error::not_found);
-  BOOST_CHECK(length == 0);
+  ASIO_CHECK(ec == asio::error::not_found);
+  ASIO_CHECK(length == 0);
 
   s.reset(read_data, sizeof(read_data));
   sb2.consume(sb2.size());
   length = asio::read_until(s, sb2, "WXY", ec);
-  BOOST_CHECK(!ec);
-  BOOST_CHECK(length == 25);
+  ASIO_CHECK(!ec);
+  ASIO_CHECK(length == 25);
 
   s.reset(read_data, sizeof(read_data));
   s.next_read_length(1);
   sb2.consume(sb2.size());
   length = asio::read_until(s, sb2, "WXY", ec);
-  BOOST_CHECK(!ec);
-  BOOST_CHECK(length == 25);
+  ASIO_CHECK(!ec);
+  ASIO_CHECK(length == 25);
 
   s.reset(read_data, sizeof(read_data));
   s.next_read_length(10);
   sb2.consume(sb2.size());
   length = asio::read_until(s, sb2, "WXY", ec);
-  BOOST_CHECK(!ec);
-  BOOST_CHECK(length == 25);
+  ASIO_CHECK(!ec);
+  ASIO_CHECK(length == 25);
 }
 
 class match_char
@@ -308,13 +298,15 @@ private:
 
 namespace asio {
   template <> struct is_match_condition<match_char>
-    : public boost::true_type {};
+  {
+    enum { value = true };
+  };
 } // namespace asio
 
 void test_match_condition_read_until()
 {
-  asio::io_service ios;
-  test_stream s(ios);
+  asio::io_context ioc;
+  test_stream s(ioc);
   asio::streambuf sb1;
   asio::streambuf sb2(25);
   asio::error_code ec;
@@ -322,79 +314,79 @@ void test_match_condition_read_until()
   s.reset(read_data, sizeof(read_data));
   sb1.consume(sb1.size());
   std::size_t length = asio::read_until(s, sb1, match_char('Z'));
-  BOOST_CHECK(length == 26);
+  ASIO_CHECK(length == 26);
 
   s.reset(read_data, sizeof(read_data));
   s.next_read_length(1);
   sb1.consume(sb1.size());
   length = asio::read_until(s, sb1, match_char('Z'));
-  BOOST_CHECK(length == 26);
+  ASIO_CHECK(length == 26);
 
   s.reset(read_data, sizeof(read_data));
   s.next_read_length(10);
   sb1.consume(sb1.size());
   length = asio::read_until(s, sb1, match_char('Z'));
-  BOOST_CHECK(length == 26);
+  ASIO_CHECK(length == 26);
 
   s.reset(read_data, sizeof(read_data));
   sb1.consume(sb1.size());
   length = asio::read_until(s, sb1, match_char('Z'), ec);
-  BOOST_CHECK(!ec);
-  BOOST_CHECK(length == 26);
+  ASIO_CHECK(!ec);
+  ASIO_CHECK(length == 26);
 
   s.reset(read_data, sizeof(read_data));
   s.next_read_length(1);
   sb1.consume(sb1.size());
   length = asio::read_until(s, sb1, match_char('Z'), ec);
-  BOOST_CHECK(!ec);
-  BOOST_CHECK(length == 26);
+  ASIO_CHECK(!ec);
+  ASIO_CHECK(length == 26);
 
   s.reset(read_data, sizeof(read_data));
   s.next_read_length(10);
   sb1.consume(sb1.size());
   length = asio::read_until(s, sb1, match_char('Z'), ec);
-  BOOST_CHECK(!ec);
-  BOOST_CHECK(length == 26);
+  ASIO_CHECK(!ec);
+  ASIO_CHECK(length == 26);
 
   s.reset(read_data, sizeof(read_data));
   sb2.consume(sb2.size());
   length = asio::read_until(s, sb2, match_char('Z'), ec);
-  BOOST_CHECK(ec == asio::error::not_found);
-  BOOST_CHECK(length == 0);
+  ASIO_CHECK(ec == asio::error::not_found);
+  ASIO_CHECK(length == 0);
 
   s.reset(read_data, sizeof(read_data));
   s.next_read_length(1);
   sb2.consume(sb2.size());
   length = asio::read_until(s, sb2, match_char('Z'), ec);
-  BOOST_CHECK(ec == asio::error::not_found);
-  BOOST_CHECK(length == 0);
+  ASIO_CHECK(ec == asio::error::not_found);
+  ASIO_CHECK(length == 0);
 
   s.reset(read_data, sizeof(read_data));
   s.next_read_length(10);
   sb2.consume(sb2.size());
   length = asio::read_until(s, sb2, match_char('Z'), ec);
-  BOOST_CHECK(ec == asio::error::not_found);
-  BOOST_CHECK(length == 0);
+  ASIO_CHECK(ec == asio::error::not_found);
+  ASIO_CHECK(length == 0);
 
   s.reset(read_data, sizeof(read_data));
   sb2.consume(sb2.size());
   length = asio::read_until(s, sb2, match_char('Y'), ec);
-  BOOST_CHECK(!ec);
-  BOOST_CHECK(length == 25);
+  ASIO_CHECK(!ec);
+  ASIO_CHECK(length == 25);
 
   s.reset(read_data, sizeof(read_data));
   s.next_read_length(1);
   sb2.consume(sb2.size());
   length = asio::read_until(s, sb2, match_char('Y'), ec);
-  BOOST_CHECK(!ec);
-  BOOST_CHECK(length == 25);
+  ASIO_CHECK(!ec);
+  ASIO_CHECK(length == 25);
 
   s.reset(read_data, sizeof(read_data));
   s.next_read_length(10);
   sb2.consume(sb2.size());
   length = asio::read_until(s, sb2, match_char('Y'), ec);
-  BOOST_CHECK(!ec);
-  BOOST_CHECK(length == 25);
+  ASIO_CHECK(!ec);
+  ASIO_CHECK(length == 25);
 }
 
 void async_read_handler(
@@ -408,8 +400,16 @@ void async_read_handler(
 
 void test_char_async_read_until()
 {
-  asio::io_service ios;
-  test_stream s(ios);
+#if defined(ASIO_HAS_BOOST_BIND)
+  namespace bindns = boost;
+#else // defined(ASIO_HAS_BOOST_BIND)
+  namespace bindns = std;
+  using std::placeholders::_1;
+  using std::placeholders::_2;
+#endif // defined(ASIO_HAS_BOOST_BIND)
+
+  asio::io_context ioc;
+  test_stream s(ioc);
   asio::streambuf sb1;
   asio::streambuf sb2(25);
   asio::error_code ec;
@@ -422,13 +422,13 @@ void test_char_async_read_until()
   called = false;
   sb1.consume(sb1.size());
   asio::async_read_until(s, sb1, 'Z',
-      boost::bind(async_read_handler, asio::placeholders::error, &ec,
-        asio::placeholders::bytes_transferred, &length, &called));
-  ios.reset();
-  ios.run();
-  BOOST_CHECK(called);
-  BOOST_CHECK(!ec);
-  BOOST_CHECK(length == 26);
+      bindns::bind(async_read_handler, _1, &ec,
+        _2, &length, &called));
+  ioc.restart();
+  ioc.run();
+  ASIO_CHECK(called);
+  ASIO_CHECK(!ec);
+  ASIO_CHECK(length == 26);
 
   s.reset(read_data, sizeof(read_data));
   s.next_read_length(1);
@@ -437,13 +437,13 @@ void test_char_async_read_until()
   called = false;
   sb1.consume(sb1.size());
   asio::async_read_until(s, sb1, 'Z',
-      boost::bind(async_read_handler, asio::placeholders::error, &ec,
-        asio::placeholders::bytes_transferred, &length, &called));
-  ios.reset();
-  ios.run();
-  BOOST_CHECK(called);
-  BOOST_CHECK(!ec);
-  BOOST_CHECK(length == 26);
+      bindns::bind(async_read_handler, _1, &ec,
+        _2, &length, &called));
+  ioc.restart();
+  ioc.run();
+  ASIO_CHECK(called);
+  ASIO_CHECK(!ec);
+  ASIO_CHECK(length == 26);
 
   s.reset(read_data, sizeof(read_data));
   s.next_read_length(10);
@@ -452,13 +452,13 @@ void test_char_async_read_until()
   called = false;
   sb1.consume(sb1.size());
   asio::async_read_until(s, sb1, 'Z',
-      boost::bind(async_read_handler, asio::placeholders::error, &ec,
-        asio::placeholders::bytes_transferred, &length, &called));
-  ios.reset();
-  ios.run();
-  BOOST_CHECK(called);
-  BOOST_CHECK(!ec);
-  BOOST_CHECK(length == 26);
+      bindns::bind(async_read_handler, _1, &ec,
+        _2, &length, &called));
+  ioc.restart();
+  ioc.run();
+  ASIO_CHECK(called);
+  ASIO_CHECK(!ec);
+  ASIO_CHECK(length == 26);
 
   s.reset(read_data, sizeof(read_data));
   ec = asio::error_code();
@@ -466,13 +466,13 @@ void test_char_async_read_until()
   called = false;
   sb2.consume(sb2.size());
   asio::async_read_until(s, sb2, 'Z',
-      boost::bind(async_read_handler, asio::placeholders::error, &ec,
-        asio::placeholders::bytes_transferred, &length, &called));
-  ios.reset();
-  ios.run();
-  BOOST_CHECK(called);
-  BOOST_CHECK(ec == asio::error::not_found);
-  BOOST_CHECK(length == 0);
+      bindns::bind(async_read_handler, _1, &ec,
+        _2, &length, &called));
+  ioc.restart();
+  ioc.run();
+  ASIO_CHECK(called);
+  ASIO_CHECK(ec == asio::error::not_found);
+  ASIO_CHECK(length == 0);
 
   s.reset(read_data, sizeof(read_data));
   s.next_read_length(1);
@@ -481,13 +481,13 @@ void test_char_async_read_until()
   called = false;
   sb2.consume(sb2.size());
   asio::async_read_until(s, sb2, 'Z',
-      boost::bind(async_read_handler, asio::placeholders::error, &ec,
-        asio::placeholders::bytes_transferred, &length, &called));
-  ios.reset();
-  ios.run();
-  BOOST_CHECK(called);
-  BOOST_CHECK(ec == asio::error::not_found);
-  BOOST_CHECK(length == 0);
+      bindns::bind(async_read_handler, _1, &ec,
+        _2, &length, &called));
+  ioc.restart();
+  ioc.run();
+  ASIO_CHECK(called);
+  ASIO_CHECK(ec == asio::error::not_found);
+  ASIO_CHECK(length == 0);
 
   s.reset(read_data, sizeof(read_data));
   s.next_read_length(10);
@@ -496,13 +496,13 @@ void test_char_async_read_until()
   called = false;
   sb2.consume(sb2.size());
   asio::async_read_until(s, sb2, 'Z',
-      boost::bind(async_read_handler, asio::placeholders::error, &ec,
-        asio::placeholders::bytes_transferred, &length, &called));
-  ios.reset();
-  ios.run();
-  BOOST_CHECK(called);
-  BOOST_CHECK(ec == asio::error::not_found);
-  BOOST_CHECK(length == 0);
+      bindns::bind(async_read_handler, _1, &ec,
+        _2, &length, &called));
+  ioc.restart();
+  ioc.run();
+  ASIO_CHECK(called);
+  ASIO_CHECK(ec == asio::error::not_found);
+  ASIO_CHECK(length == 0);
 
   s.reset(read_data, sizeof(read_data));
   ec = asio::error_code();
@@ -510,13 +510,13 @@ void test_char_async_read_until()
   called = false;
   sb2.consume(sb2.size());
   asio::async_read_until(s, sb2, 'Y',
-      boost::bind(async_read_handler, asio::placeholders::error, &ec,
-        asio::placeholders::bytes_transferred, &length, &called));
-  ios.reset();
-  ios.run();
-  BOOST_CHECK(called);
-  BOOST_CHECK(!ec);
-  BOOST_CHECK(length == 25);
+      bindns::bind(async_read_handler, _1, &ec,
+        _2, &length, &called));
+  ioc.restart();
+  ioc.run();
+  ASIO_CHECK(called);
+  ASIO_CHECK(!ec);
+  ASIO_CHECK(length == 25);
 
   s.reset(read_data, sizeof(read_data));
   s.next_read_length(1);
@@ -525,13 +525,13 @@ void test_char_async_read_until()
   called = false;
   sb2.consume(sb2.size());
   asio::async_read_until(s, sb2, 'Y',
-      boost::bind(async_read_handler, asio::placeholders::error, &ec,
-        asio::placeholders::bytes_transferred, &length, &called));
-  ios.reset();
-  ios.run();
-  BOOST_CHECK(called);
-  BOOST_CHECK(!ec);
-  BOOST_CHECK(length == 25);
+      bindns::bind(async_read_handler, _1, &ec,
+        _2, &length, &called));
+  ioc.restart();
+  ioc.run();
+  ASIO_CHECK(called);
+  ASIO_CHECK(!ec);
+  ASIO_CHECK(length == 25);
 
   s.reset(read_data, sizeof(read_data));
   s.next_read_length(10);
@@ -540,19 +540,35 @@ void test_char_async_read_until()
   called = false;
   sb2.consume(sb2.size());
   asio::async_read_until(s, sb2, 'Y',
-      boost::bind(async_read_handler, asio::placeholders::error, &ec,
-        asio::placeholders::bytes_transferred, &length, &called));
-  ios.reset();
-  ios.run();
-  BOOST_CHECK(called);
-  BOOST_CHECK(!ec);
-  BOOST_CHECK(length == 25);
+      bindns::bind(async_read_handler, _1, &ec,
+        _2, &length, &called));
+  ioc.restart();
+  ioc.run();
+  ASIO_CHECK(called);
+  ASIO_CHECK(!ec);
+  ASIO_CHECK(length == 25);
+
+  s.reset(read_data, sizeof(read_data));
+  sb2.consume(sb2.size());
+  int i = asio::async_read_until(s, sb2, 'Y',
+      archetypes::lazy_handler());
+  ASIO_CHECK(i == 42);
+  ioc.restart();
+  ioc.run();
 }
 
 void test_string_async_read_until()
 {
-  asio::io_service ios;
-  test_stream s(ios);
+#if defined(ASIO_HAS_BOOST_BIND)
+  namespace bindns = boost;
+#else // defined(ASIO_HAS_BOOST_BIND)
+  namespace bindns = std;
+  using std::placeholders::_1;
+  using std::placeholders::_2;
+#endif // defined(ASIO_HAS_BOOST_BIND)
+
+  asio::io_context ioc;
+  test_stream s(ioc);
   asio::streambuf sb1;
   asio::streambuf sb2(25);
   asio::error_code ec;
@@ -565,13 +581,13 @@ void test_string_async_read_until()
   called = false;
   sb1.consume(sb1.size());
   asio::async_read_until(s, sb1, "XYZ",
-      boost::bind(async_read_handler, asio::placeholders::error, &ec,
-        asio::placeholders::bytes_transferred, &length, &called));
-  ios.reset();
-  ios.run();
-  BOOST_CHECK(called);
-  BOOST_CHECK(!ec);
-  BOOST_CHECK(length == 26);
+      bindns::bind(async_read_handler, _1, &ec,
+        _2, &length, &called));
+  ioc.restart();
+  ioc.run();
+  ASIO_CHECK(called);
+  ASIO_CHECK(!ec);
+  ASIO_CHECK(length == 26);
 
   s.reset(read_data, sizeof(read_data));
   s.next_read_length(1);
@@ -580,13 +596,13 @@ void test_string_async_read_until()
   called = false;
   sb1.consume(sb1.size());
   asio::async_read_until(s, sb1, "XYZ",
-      boost::bind(async_read_handler, asio::placeholders::error, &ec,
-        asio::placeholders::bytes_transferred, &length, &called));
-  ios.reset();
-  ios.run();
-  BOOST_CHECK(called);
-  BOOST_CHECK(!ec);
-  BOOST_CHECK(length == 26);
+      bindns::bind(async_read_handler, _1, &ec,
+        _2, &length, &called));
+  ioc.restart();
+  ioc.run();
+  ASIO_CHECK(called);
+  ASIO_CHECK(!ec);
+  ASIO_CHECK(length == 26);
 
   s.reset(read_data, sizeof(read_data));
   s.next_read_length(10);
@@ -595,13 +611,13 @@ void test_string_async_read_until()
   called = false;
   sb1.consume(sb1.size());
   asio::async_read_until(s, sb1, "XYZ",
-      boost::bind(async_read_handler, asio::placeholders::error, &ec,
-        asio::placeholders::bytes_transferred, &length, &called));
-  ios.reset();
-  ios.run();
-  BOOST_CHECK(called);
-  BOOST_CHECK(!ec);
-  BOOST_CHECK(length == 26);
+      bindns::bind(async_read_handler, _1, &ec,
+        _2, &length, &called));
+  ioc.restart();
+  ioc.run();
+  ASIO_CHECK(called);
+  ASIO_CHECK(!ec);
+  ASIO_CHECK(length == 26);
 
   s.reset(read_data, sizeof(read_data));
   ec = asio::error_code();
@@ -609,13 +625,13 @@ void test_string_async_read_until()
   called = false;
   sb2.consume(sb2.size());
   asio::async_read_until(s, sb2, "XYZ",
-      boost::bind(async_read_handler, asio::placeholders::error, &ec,
-        asio::placeholders::bytes_transferred, &length, &called));
-  ios.reset();
-  ios.run();
-  BOOST_CHECK(called);
-  BOOST_CHECK(ec == asio::error::not_found);
-  BOOST_CHECK(length == 0);
+      bindns::bind(async_read_handler, _1, &ec,
+        _2, &length, &called));
+  ioc.restart();
+  ioc.run();
+  ASIO_CHECK(called);
+  ASIO_CHECK(ec == asio::error::not_found);
+  ASIO_CHECK(length == 0);
 
   s.reset(read_data, sizeof(read_data));
   s.next_read_length(1);
@@ -624,13 +640,13 @@ void test_string_async_read_until()
   called = false;
   sb2.consume(sb2.size());
   asio::async_read_until(s, sb2, "XYZ",
-      boost::bind(async_read_handler, asio::placeholders::error, &ec,
-        asio::placeholders::bytes_transferred, &length, &called));
-  ios.reset();
-  ios.run();
-  BOOST_CHECK(called);
-  BOOST_CHECK(ec == asio::error::not_found);
-  BOOST_CHECK(length == 0);
+      bindns::bind(async_read_handler, _1, &ec,
+        _2, &length, &called));
+  ioc.restart();
+  ioc.run();
+  ASIO_CHECK(called);
+  ASIO_CHECK(ec == asio::error::not_found);
+  ASIO_CHECK(length == 0);
 
   s.reset(read_data, sizeof(read_data));
   s.next_read_length(10);
@@ -639,13 +655,13 @@ void test_string_async_read_until()
   called = false;
   sb2.consume(sb2.size());
   asio::async_read_until(s, sb2, "XYZ",
-      boost::bind(async_read_handler, asio::placeholders::error, &ec,
-        asio::placeholders::bytes_transferred, &length, &called));
-  ios.reset();
-  ios.run();
-  BOOST_CHECK(called);
-  BOOST_CHECK(ec == asio::error::not_found);
-  BOOST_CHECK(length == 0);
+      bindns::bind(async_read_handler, _1, &ec,
+        _2, &length, &called));
+  ioc.restart();
+  ioc.run();
+  ASIO_CHECK(called);
+  ASIO_CHECK(ec == asio::error::not_found);
+  ASIO_CHECK(length == 0);
 
   s.reset(read_data, sizeof(read_data));
   ec = asio::error_code();
@@ -653,13 +669,13 @@ void test_string_async_read_until()
   called = false;
   sb2.consume(sb2.size());
   asio::async_read_until(s, sb2, "WXY",
-      boost::bind(async_read_handler, asio::placeholders::error, &ec,
-        asio::placeholders::bytes_transferred, &length, &called));
-  ios.reset();
-  ios.run();
-  BOOST_CHECK(called);
-  BOOST_CHECK(!ec);
-  BOOST_CHECK(length == 25);
+      bindns::bind(async_read_handler, _1, &ec,
+        _2, &length, &called));
+  ioc.restart();
+  ioc.run();
+  ASIO_CHECK(called);
+  ASIO_CHECK(!ec);
+  ASIO_CHECK(length == 25);
 
   s.reset(read_data, sizeof(read_data));
   s.next_read_length(1);
@@ -668,13 +684,13 @@ void test_string_async_read_until()
   called = false;
   sb2.consume(sb2.size());
   asio::async_read_until(s, sb2, "WXY",
-      boost::bind(async_read_handler, asio::placeholders::error, &ec,
-        asio::placeholders::bytes_transferred, &length, &called));
-  ios.reset();
-  ios.run();
-  BOOST_CHECK(called);
-  BOOST_CHECK(!ec);
-  BOOST_CHECK(length == 25);
+      bindns::bind(async_read_handler, _1, &ec,
+        _2, &length, &called));
+  ioc.restart();
+  ioc.run();
+  ASIO_CHECK(called);
+  ASIO_CHECK(!ec);
+  ASIO_CHECK(length == 25);
 
   s.reset(read_data, sizeof(read_data));
   s.next_read_length(10);
@@ -683,19 +699,35 @@ void test_string_async_read_until()
   called = false;
   sb2.consume(sb2.size());
   asio::async_read_until(s, sb2, "WXY",
-      boost::bind(async_read_handler, asio::placeholders::error, &ec,
-        asio::placeholders::bytes_transferred, &length, &called));
-  ios.reset();
-  ios.run();
-  BOOST_CHECK(called);
-  BOOST_CHECK(!ec);
-  BOOST_CHECK(length == 25);
+      bindns::bind(async_read_handler, _1, &ec,
+        _2, &length, &called));
+  ioc.restart();
+  ioc.run();
+  ASIO_CHECK(called);
+  ASIO_CHECK(!ec);
+  ASIO_CHECK(length == 25);
+
+  s.reset(read_data, sizeof(read_data));
+  sb2.consume(sb2.size());
+  int i = asio::async_read_until(s, sb2, "WXY",
+      archetypes::lazy_handler());
+  ASIO_CHECK(i == 42);
+  ioc.restart();
+  ioc.run();
 }
 
 void test_match_condition_async_read_until()
 {
-  asio::io_service ios;
-  test_stream s(ios);
+#if defined(ASIO_HAS_BOOST_BIND)
+  namespace bindns = boost;
+#else // defined(ASIO_HAS_BOOST_BIND)
+  namespace bindns = std;
+  using std::placeholders::_1;
+  using std::placeholders::_2;
+#endif // defined(ASIO_HAS_BOOST_BIND)
+
+  asio::io_context ioc;
+  test_stream s(ioc);
   asio::streambuf sb1;
   asio::streambuf sb2(25);
   asio::error_code ec;
@@ -708,13 +740,13 @@ void test_match_condition_async_read_until()
   called = false;
   sb1.consume(sb1.size());
   asio::async_read_until(s, sb1, match_char('Z'),
-      boost::bind(async_read_handler, asio::placeholders::error, &ec,
-        asio::placeholders::bytes_transferred, &length, &called));
-  ios.reset();
-  ios.run();
-  BOOST_CHECK(called);
-  BOOST_CHECK(!ec);
-  BOOST_CHECK(length == 26);
+      bindns::bind(async_read_handler, _1, &ec,
+        _2, &length, &called));
+  ioc.restart();
+  ioc.run();
+  ASIO_CHECK(called);
+  ASIO_CHECK(!ec);
+  ASIO_CHECK(length == 26);
 
   s.reset(read_data, sizeof(read_data));
   s.next_read_length(1);
@@ -723,13 +755,13 @@ void test_match_condition_async_read_until()
   called = false;
   sb1.consume(sb1.size());
   asio::async_read_until(s, sb1, match_char('Z'),
-      boost::bind(async_read_handler, asio::placeholders::error, &ec,
-        asio::placeholders::bytes_transferred, &length, &called));
-  ios.reset();
-  ios.run();
-  BOOST_CHECK(called);
-  BOOST_CHECK(!ec);
-  BOOST_CHECK(length == 26);
+      bindns::bind(async_read_handler, _1, &ec,
+        _2, &length, &called));
+  ioc.restart();
+  ioc.run();
+  ASIO_CHECK(called);
+  ASIO_CHECK(!ec);
+  ASIO_CHECK(length == 26);
 
   s.reset(read_data, sizeof(read_data));
   s.next_read_length(10);
@@ -738,13 +770,13 @@ void test_match_condition_async_read_until()
   called = false;
   sb1.consume(sb1.size());
   asio::async_read_until(s, sb1, match_char('Z'),
-      boost::bind(async_read_handler, asio::placeholders::error, &ec,
-        asio::placeholders::bytes_transferred, &length, &called));
-  ios.reset();
-  ios.run();
-  BOOST_CHECK(called);
-  BOOST_CHECK(!ec);
-  BOOST_CHECK(length == 26);
+      bindns::bind(async_read_handler, _1, &ec,
+        _2, &length, &called));
+  ioc.restart();
+  ioc.run();
+  ASIO_CHECK(called);
+  ASIO_CHECK(!ec);
+  ASIO_CHECK(length == 26);
 
   s.reset(read_data, sizeof(read_data));
   ec = asio::error_code();
@@ -752,13 +784,13 @@ void test_match_condition_async_read_until()
   called = false;
   sb2.consume(sb2.size());
   asio::async_read_until(s, sb2, match_char('Z'),
-      boost::bind(async_read_handler, asio::placeholders::error, &ec,
-        asio::placeholders::bytes_transferred, &length, &called));
-  ios.reset();
-  ios.run();
-  BOOST_CHECK(called);
-  BOOST_CHECK(ec == asio::error::not_found);
-  BOOST_CHECK(length == 0);
+      bindns::bind(async_read_handler, _1, &ec,
+        _2, &length, &called));
+  ioc.restart();
+  ioc.run();
+  ASIO_CHECK(called);
+  ASIO_CHECK(ec == asio::error::not_found);
+  ASIO_CHECK(length == 0);
 
   s.reset(read_data, sizeof(read_data));
   s.next_read_length(1);
@@ -767,13 +799,13 @@ void test_match_condition_async_read_until()
   called = false;
   sb2.consume(sb2.size());
   asio::async_read_until(s, sb2, match_char('Z'),
-      boost::bind(async_read_handler, asio::placeholders::error, &ec,
-        asio::placeholders::bytes_transferred, &length, &called));
-  ios.reset();
-  ios.run();
-  BOOST_CHECK(called);
-  BOOST_CHECK(ec == asio::error::not_found);
-  BOOST_CHECK(length == 0);
+      bindns::bind(async_read_handler, _1, &ec,
+        _2, &length, &called));
+  ioc.restart();
+  ioc.run();
+  ASIO_CHECK(called);
+  ASIO_CHECK(ec == asio::error::not_found);
+  ASIO_CHECK(length == 0);
 
   s.reset(read_data, sizeof(read_data));
   s.next_read_length(10);
@@ -782,13 +814,13 @@ void test_match_condition_async_read_until()
   called = false;
   sb2.consume(sb2.size());
   asio::async_read_until(s, sb2, match_char('Z'),
-      boost::bind(async_read_handler, asio::placeholders::error, &ec,
-        asio::placeholders::bytes_transferred, &length, &called));
-  ios.reset();
-  ios.run();
-  BOOST_CHECK(called);
-  BOOST_CHECK(ec == asio::error::not_found);
-  BOOST_CHECK(length == 0);
+      bindns::bind(async_read_handler, _1, &ec,
+        _2, &length, &called));
+  ioc.restart();
+  ioc.run();
+  ASIO_CHECK(called);
+  ASIO_CHECK(ec == asio::error::not_found);
+  ASIO_CHECK(length == 0);
 
   s.reset(read_data, sizeof(read_data));
   ec = asio::error_code();
@@ -796,13 +828,13 @@ void test_match_condition_async_read_until()
   called = false;
   sb2.consume(sb2.size());
   asio::async_read_until(s, sb2, match_char('Y'),
-      boost::bind(async_read_handler, asio::placeholders::error, &ec,
-        asio::placeholders::bytes_transferred, &length, &called));
-  ios.reset();
-  ios.run();
-  BOOST_CHECK(called);
-  BOOST_CHECK(!ec);
-  BOOST_CHECK(length == 25);
+      bindns::bind(async_read_handler, _1, &ec,
+        _2, &length, &called));
+  ioc.restart();
+  ioc.run();
+  ASIO_CHECK(called);
+  ASIO_CHECK(!ec);
+  ASIO_CHECK(length == 25);
 
   s.reset(read_data, sizeof(read_data));
   s.next_read_length(1);
@@ -811,13 +843,13 @@ void test_match_condition_async_read_until()
   called = false;
   sb2.consume(sb2.size());
   asio::async_read_until(s, sb2, match_char('Y'),
-      boost::bind(async_read_handler, asio::placeholders::error, &ec,
-        asio::placeholders::bytes_transferred, &length, &called));
-  ios.reset();
-  ios.run();
-  BOOST_CHECK(called);
-  BOOST_CHECK(!ec);
-  BOOST_CHECK(length == 25);
+      bindns::bind(async_read_handler, _1, &ec,
+        _2, &length, &called));
+  ioc.restart();
+  ioc.run();
+  ASIO_CHECK(called);
+  ASIO_CHECK(!ec);
+  ASIO_CHECK(length == 25);
 
   s.reset(read_data, sizeof(read_data));
   s.next_read_length(10);
@@ -826,23 +858,30 @@ void test_match_condition_async_read_until()
   called = false;
   sb2.consume(sb2.size());
   asio::async_read_until(s, sb2, match_char('Y'),
-      boost::bind(async_read_handler, asio::placeholders::error, &ec,
-        asio::placeholders::bytes_transferred, &length, &called));
-  ios.reset();
-  ios.run();
-  BOOST_CHECK(called);
-  BOOST_CHECK(!ec);
-  BOOST_CHECK(length == 25);
+      bindns::bind(async_read_handler, _1, &ec,
+        _2, &length, &called));
+  ioc.restart();
+  ioc.run();
+  ASIO_CHECK(called);
+  ASIO_CHECK(!ec);
+  ASIO_CHECK(length == 25);
+
+  s.reset(read_data, sizeof(read_data));
+  sb2.consume(sb2.size());
+  int i = asio::async_read_until(s, sb2, match_char('Y'),
+      archetypes::lazy_handler());
+  ASIO_CHECK(i == 42);
+  ioc.restart();
+  ioc.run();
 }
 
-test_suite* init_unit_test_suite(int, char*[])
-{
-  test_suite* test = BOOST_TEST_SUITE("read_until");
-  test->add(BOOST_TEST_CASE(&test_char_read_until));
-  test->add(BOOST_TEST_CASE(&test_string_read_until));
-  test->add(BOOST_TEST_CASE(&test_match_condition_read_until));
-  test->add(BOOST_TEST_CASE(&test_char_async_read_until));
-  test->add(BOOST_TEST_CASE(&test_string_async_read_until));
-  test->add(BOOST_TEST_CASE(&test_match_condition_async_read_until));
-  return test;
-}
+ASIO_TEST_SUITE
+(
+  "read_until",
+  ASIO_TEST_CASE(test_char_read_until)
+  ASIO_TEST_CASE(test_string_read_until)
+  ASIO_TEST_CASE(test_match_condition_read_until)
+  ASIO_TEST_CASE(test_char_async_read_until)
+  ASIO_TEST_CASE(test_string_async_read_until)
+  ASIO_TEST_CASE(test_match_condition_async_read_until)
+)
